"use strict";

exports.__esModule = true;
exports.default = compileImageParameter;

var _utils = require("../utils");

// http://cloudinary.com/documentation/image_transformation_reference#format_parameter
var formatOptions = ['jpg', 'jpe', 'jpeg', 'jpc', 'jp2', 'j2k', 'wdp', 'jxr', 'hdp', 'png', 'gif', 'webp', 'bmp', 'tif', 'tiff', 'ico', 'pdf', 'ps', 'ept', 'eps', 'eps3', 'psd', 'svg', 'ai', 'djvu', 'flif']; // http://cloudinary.com/documentation/image_transformation_reference#crop_parameter

var cropOptions = ['scale', 'fit', 'limit', 'mfit', 'fill', 'lfill', 'pad', 'lpad', 'mpad', 'crop', 'thumb', 'imagga_crop', 'imagga_scale']; // http://cloudinary.com/documentation/image_transformation_reference#gravity_parameter

var gravityOptions = ['north_west', 'north', 'north_east', 'west', 'center', 'east', 'south_west', 'south', 'south_east', 'xy_center', 'face', 'face:center', 'face:auto', 'faces', 'faces:center', 'faces:auto', 'body', 'body:face', 'ocr_text', 'adv_face', 'adv_faces', 'adv_eyes', 'custom', 'custom:face', 'custom:faces', 'custom:adv_face', 'custom:adv_faces']; // http://cloudinary.com/documentation/image_transformation_reference#quality_parameter

var qualityOptions = ['auto', 'auto:best', 'auto:good', 'auto:eco', 'auto:low', 'jpegmini']; // http://cloudinary.com/documentation/image_transformation_reference#angle_parameter

var angleOptions = ['auto_right', 'auto_left', 'ignore', 'vflip', 'hflip']; // http://cloudinary.com/documentation/image_transformation_reference#background_parameter

var backgroundOptions = ['auto:border', 'auto:predominant', 'auto:border_contrast', 'auto:predominant_contrast']; // http://cloudinary.com/documentation/image_transformation_reference#color_parameter
// A color name or rgb: followed by an rgb[a] or rrggbb[aa] hex value

var colorRegex = /^(?:[a-z]+$|rgb:(?:[0-9a-f]{3,4}$|[0-9a-f]{6}$|[0-9a-f]{8}$))/i; // http://cloudinary.com/documentation/image_transformation_reference#color_space_parameter

var colorSpaceOptions = ['srgb', 'tinysrgb', 'no_cmyk'];
var flagOptions = ['any_format', 'attachment', 'apng', 'awebp', 'clip', 'clip_evenodd', 'cutter', 'force_strip', 'ignore_aspect_ratio', 'immutable_cache', 'keep_attribution', 'keep_iptc', 'layer_apply', 'lossy', 'no_overflow', 'preserve_transparency', 'png8', 'png24', 'png32', 'progressive', 'progressive:semi', 'progressive:steep', 'progressive:none', 'rasterize', 'region_relative', 'relative', 'strip_profile', 'text_no_trim', 'tiff8_lwz', 'tiled'];
var validate = {
  width: function width(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) || value === 'auto' || (0, _utils.startsWith)('auto:', value), 'width', value, "should be a number, 'auto', or a string starting with 'auto:'");
  },
  height: function height(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value), 'height', value, 'should be a number');
  },
  crop: function crop(value) {
    return (0, _utils.invariantImage)((0, _utils.includes)(value, cropOptions), 'crop', value, (0, _utils.shouldBeOneOf)(cropOptions));
  },
  aspectRatio: function aspectRatio(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) || value.match(/^\d+:\d+$/), 'aspectRatio', value, 'should be a number or have the form x:y');
  },
  gravity: function gravity(value) {
    return (0, _utils.invariantImage)((0, _utils.includes)(value, gravityOptions) || value === 'auto' || (0, _utils.startsWith)('auto:', value), 'gravity', value, (0, _utils.shouldBeOneOf)(gravityOptions) + ", 'auto', or a string starting with 'auto:'");
  },
  zoom: function zoom(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value), 'zoom', value, 'should be a number');
  },
  x: function x(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value), 'x', value, 'should be a number');
  },
  y: function y(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value), 'y', value, 'should be a number');
  },
  fetchFormat: function fetchFormat(value) {
    return (0, _utils.invariantImage)((0, _utils.includes)(value, ['auto'].concat(formatOptions)), 'fetchFormat', value, (0, _utils.shouldBeOneOf)(['auto'].concat(formatOptions)) + " or 'auto'");
  },
  quality: function quality(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) && 1 <= +value && +value <= 100 || typeof value === 'string' && value.match(/^\d+:\d+$/) || (0, _utils.includes)(value, qualityOptions), 'quality', value, (0, _utils.shouldBeOneOf)(formatOptions) + ", a number between 1 and 100, or have the form x:y");
  },
  radius: function radius(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) || value === 'max' || value.match(/^\d+(?::\d+){0,3}$/), 'radius', value, "should be a number, 'max' or have the form x[:y[:z[:u]]]");
  },
  angle: function angle(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) || (0, _utils.includes)(value, angleOptions), 'angle', value, (0, _utils.shouldBeOneOf)(angleOptions) + " or a number");
  },
  effect: function effect(value) {},
  opacity: function opacity(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) && 0 <= +value && +value <= 100, 'opacity', value, 'should be a number between 0 and 100');
  },
  border: function border(value) {
    return (0, _utils.invariantImage)(value.match(new RegExp("^\\d+px_solid_" + colorRegex.toString().replace(/[/^i]/g, '') + "$", 'i')), 'border', value, "should be an object with 'width' & 'color' or a string of the form 'width_style_color'");
  },
  borderObjectColor: function borderObjectColor(value) {
    return (0, _utils.invariantImage)('color' in value, 'border', JSON.stringify(value), "should contain a 'color' property when provided as an object");
  },
  background: function background(value) {
    return (0, _utils.invariantImage)(typeof value === 'string' && ((0, _utils.includes)(value, backgroundOptions) || value.match(colorRegex)), 'background', value, (0, _utils.shouldBeOneOf)(backgroundOptions) + " or a color");
  },
  overlay: function overlay(value) {},
  underlay: function underlay(value) {},
  textCaptionStyles: function textCaptionStyles(value) {
    var fontFamily = value.fontFamily,
        fontSize = value.fontSize,
        fontWeight = value.fontWeight,
        fontStyle = value.fontStyle,
        textDecoration = value.textDecoration,
        textAlign = value.textAlign,
        stroke = value.stroke,
        letterSpacing = value.letterSpacing,
        lineSpacing = value.lineSpacing;
    (0, _utils.invariantImage)(fontFamily && fontSize && (0, _utils.isNumber)(fontSize), 'text caption', JSON.stringify(value), "required options are 'fontFamily' and 'fontSize'", '/image_transformations#adding_text_captions');
    (0, _utils.invariantImage)((!fontWeight || (0, _utils.includes)(fontWeight, ['normal', 'bold'])) && (!fontStyle || (0, _utils.includes)(fontStyle, ['normal', 'italic'])) && (!textDecoration || (0, _utils.includes)(textDecoration, ['none', 'underline', 'strikethrough'])) && (!textAlign || (0, _utils.includes)(textAlign, ['left', 'center', 'right', 'start', 'end', 'justify'])) && (!stroke || (0, _utils.includes)(stroke, ['none', 'stroke'])) && (!letterSpacing || (0, _utils.isNumber)(letterSpacing)) && (!lineSpacing || (0, _utils.isNumber)(lineSpacing)), 'text caption', JSON.stringify(value), "options are invalid", '/image_transformations#adding_text_captions');
  },
  defaultImage: function defaultImage(value) {
    return (0, _utils.invariantImage)(value.match(/^[\w+-]+\.[\w]{3,4}$/) && (0, _utils.includes)(value.substr(value.indexOf('.') + 1), formatOptions), 'defaultImage', value, "must include a file extension which " + (0, _utils.shouldBeOneOf)(formatOptions));
  },
  delay: function delay(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) && value >= 0, 'delay', value, "must be a positive number");
  },
  color: function color(value) {
    return (0, _utils.invariantImage)(value.match(colorRegex), 'color', value, "must be a named color, short #rgb[a] or long #rrggbb[aa] hex value");
  },
  colorSpace: function colorSpace(value) {
    return (0, _utils.invariantImage)((0, _utils.includes)(value, colorSpaceOptions) || value.match(/^icc:[\w+-]+\.[\w]{3,4}$/), 'colorSpace', value, (0, _utils.shouldBeOneOf)(colorSpaceOptions) + " or 'icc:(public_id)'");
  },
  dpr: function dpr(value) {
    return (0, _utils.invariantImage)(value === 'auto' || (0, _utils.isNumber)(value) && value > 0, 'dpr', value, 'should be `auto` or a number greater than 0');
  },
  page: function page(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) && value > 0, 'page', value, 'should be an integer greater than 0');
  },
  density: function density(value) {
    return (0, _utils.invariantImage)((0, _utils.isNumber)(value) && 0 < value && value <= 300, 'density', value, 'should be a number greater than 0 up to 300');
  },
  flags: function flags(value) {
    return (0, _utils.invariantImage)(value.split('.').every(function (flag) {
      return (0, _utils.includes)(flag, flagOptions);
    }), 'flags', JSON.stringify(value), (0, _utils.shouldBeOneOf)(flagOptions) + ", an array of options or '.' separated options");
  }
};
var urlParameters = {
  width: 'w_',
  height: 'h_',
  crop: 'c_',
  aspectRatio: 'ar_',
  gravity: 'g_',
  zoom: 'z_',
  x: 'x_',
  y: 'y_',
  fetchFormat: 'f_',
  quality: 'q_',
  radius: 'r_',
  angle: 'a_',
  effect: 'e_',
  opacity: 'o_',
  border: 'bo_',
  background: 'b_',
  overlay: 'l_',
  underlay: 'u_',
  defaultImage: 'd_',
  delay: 'dl_',
  color: 'co_',
  colorSpace: 'cs_',
  dpr: 'dpr_',
  page: 'pg_',
  density: 'dn_',
  flags: 'fl_'
};

function compileImageParameter(parameter, value) {
  if (value === null) {
    return false;
  }

  switch (parameter) {
    case 'flags':
      value = Array.isArray(value) ? value.join('.') : value;
      break;

    case 'overlay':
    case 'underlay':
      if (typeof value === 'object') {
        var stringStyle = value.publicId;

        if (!stringStyle) {
          if (process.env.NODE_ENV !== 'production') {
            validate.textCaptionStyles(value);
          }

          var _value = value,
              letterSpacing = _value.letterSpacing,
              lineSpacing = _value.lineSpacing;
          stringStyle = [encodeURIComponent(value.fontFamily), value.fontSize, value.fontWeight, value.fontStyle, value.textDecoration, value.textAlign, value.stroke, letterSpacing ? "letter_spacing_" + letterSpacing : 0, lineSpacing ? "line_spacing_" + lineSpacing : 0].filter(function (option) {
            return option && option !== 'normal' && option !== 'none';
          }).join('_');
        }

        value = "text:" + stringStyle + ":" + encodeURIComponent(value.text);
      }

      break;

    case 'border':
      if (typeof value === 'object') {
        if (process.env.NODE_ENV !== 'production') {
          validate.borderObjectColor(value);
        }

        if (typeof value.width !== 'string' || value.width.substr(value.width.length - 2, 2) !== 'px') {
          value.width = (value.width || 1) + 'px';
        }

        value = value.width + '_solid_' + value.color;
      }

      value = value.toLowerCase();
    // falls through

    case 'color':
    case 'background':
      value = value.replace('#', 'rgb:');
    // no default
  }

  if (process.env.NODE_ENV !== 'production') {
    if (!validate[parameter]) {
      throw new Error("Cloudinary Image :: unknown transform parameter provided: '" + parameter + "'");
    }

    validate[parameter](value);
  }

  return urlParameters[parameter] + value;
}